import numpy as np
import cv2
import os
import rasterio
import matplotlib.pyplot as plt
from Orange.widgets import widget, gui
from Orange.widgets.widget import Output, MultiInput
from PyQt5.QtGui import QPixmap
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QGraphicsScene, QGraphicsPixmapItem, QGraphicsView
from Orange.widgets import settings
from skimage.exposure import match_histograms
from drb.drivers.file import DrbFileFactory
from drb.drivers.image import DrbImageFactory, DrbImageBaseNode


class OWHistogram(widget.OWWidget):
    name = "EO Histogram"
    description = "Visualize histogram of an image" \
                  " and performs contrast adjustment"
    icon = "icons/histogram.svg"
    priority = 60

    input_nodes = []
    output_node = None

    clip_limit = settings.Setting(2)

    want_main_area = True
    graph_name = "thumbnailView"
    auto_commit = settings.Setting(True)

    class Inputs:
        data = MultiInput("List[DrbImageBaseNode]",
                          DrbImageBaseNode, auto_summary=False)

    class Outputs:
        output = Output("DrbImageBaseNode",
                        DrbImageBaseNode, auto_summary=False)

    def __init__(self):
        super().__init__()
        self.output = None

        self.combobox_node = gui.comboBox(
            self.controlArea, self, "",
            box="Select Input Image:",
            contentsLength=12,
        )

        self.combobox_node.activated.connect(self._show_histogram)
        gui.rubber(self.controlArea)

        self.histogram_box = gui.widgetBox(self.controlArea, " ")
        gui.rubber(self.controlArea)

        self.basic_button = gui.button(
            self.histogram_box, self, "Basic Hist Equalization",
            callback=self.basic_hist_equalize
        )

        self.slider = gui.hSlider(
            self.histogram_box, self, "clip_limit",
            label="Select Clip Limit:", minValue=1, maxValue=100, step=1,
            orientation=Qt.Horizontal,
            width=200,
        )

        self.adaptive_button = gui.button(
            self.histogram_box, self, "Adaptive Hist Equalization",
            callback=self.adaptive_hist_equalize
        )

        self.match_box = gui.widgetBox(self.controlArea, " ")
        gui.rubber(self.controlArea)

        self.combobox_reference = gui.comboBox(
            self.match_box, self, "",
            label="Select Reference Image:",
            contentsLength=12,
        )
        self.combobox_to_match = gui.comboBox(
            self.match_box, self, "",
            label="Select Image to Match:",
            contentsLength=12,
        )

        self.match_button = gui.button(
            self.match_box, self, "Match Histograms",
            callback=self.match_histograms
        )

        gui.rubber(self.controlArea)
        gui.auto_commit(self.buttonsArea, self,
                        "auto_commit", "Send", box=False)

        gui.vBox(self.mainArea)
        self.box_scene = QGraphicsScene(self)
        self.box_view = QGraphicsView(self.box_scene)

        self.layout = self.mainArea.layout()
        self.layout.addWidget(self.box_view)

        self.mainArea.setMinimumWidth(100)

    def match_histograms(self):
        """
        Matches histograms of the images, one is chosen to be reference.
        """
        ds, filename, path = self.__read_image(self.combobox_to_match)
        profile = ds.meta.copy()
        image_to_match = ds.read()
        image_to_match = np.moveaxis(image_to_match, 0, -1)

        ds, x, x = self.__read_image(self.combobox_reference)
        ref_image = ds.read()
        ref_image = np.moveaxis(ref_image, 0, -1)

        matched_image = match_histograms(image_to_match,
                                         ref_image, channel_axis=2)
        # matched_image = rescale_intensity(matched_image, out_range=(1, 255))

        with rasterio.open(os.path.join(
                path + f'/{filename}.tif'), 'w', **profile) as dst:
            dst.write(np.moveaxis(matched_image, -1, 0))

        hist_size = 2 ** 8 - 1
        hist_range = (1, 2 ** 8 - 1)

        h1 = cv2.calcHist([ref_image], [0], None, [hist_size], hist_range)
        h2 = cv2.calcHist([matched_image], [0], None, [hist_size], hist_range)

        self.__plot([h1, h2], path, filename)

        self.output = _load_image(os.path.join(path + f'/{filename}.tif'))
        print(self.output)
        self.commit.deferred()

    def basic_hist_equalize(self):

        ds, filename, path = self.__read_image(self.combobox_node)
        profile = ds.meta.copy()
        image = ds.read()

        # convert image from RGB to HSV
        image = np.moveaxis(image, 0, -1)
        img_hsv = cv2.cvtColor(image, cv2.COLOR_RGB2HSV)

        # Histogram equalisation on the V-channel
        img_hsv[:, :, 2] = cv2.equalizeHist(img_hsv[:, :, 2])

        # convert image back from HSV to RGB
        equalized_image = cv2.cvtColor(img_hsv, cv2.COLOR_HSV2RGB)

        with rasterio.open(os.path.join(
                path + f'/{filename}.tif'), 'w', **profile) as dst:
            dst.write(np.moveaxis(equalized_image, -1, 0))

        hist_size = 2 ** 8 - 1
        hist_range = (1, 2 ** 8 - 1)

        h1 = cv2.calcHist([image], [0], None, [hist_size], hist_range)
        h2 = cv2.calcHist([equalized_image], [0], None,
                          [hist_size], hist_range)

        self.__plot([h1, h2], path, filename)

        self.output = _load_image(os.path.join(path + f'/{filename}.tif'))
        self.commit.deferred()

    def adaptive_hist_equalize(self):
        """ CLAHE is a variant of Adaptive histogram
        equalization (AHE) which takes care of over-amplification of
        the contrast. CLAHE operates on small regions in the image,
        called tiles, rather than the entire image.
        The neighboring tiles are then combined using bi-linear
        interpolation to remove the artificial boundaries.
        This algorithm can be applied to improve the contrast of images.
        We can also apply CLAHE to color images, where usually it is
        applied on the luminance channel and the results after equalizing
        only the luminance channel of an HSV image
        are much better than equalizing all the channels of the BGR image """

        ds, filename, path = self.__read_image(self.combobox_node)
        profile = ds.meta.copy()
        image = ds.read()

        # convert image from RGB to LAB
        image = np.moveaxis(image, 0, -1)
        img_lab = cv2.cvtColor(image, cv2.COLOR_RGB2LAB)

        # Applying CLAHE to L-channel
        # the risk is to over-amplify noise in relatively homogeneous
        # regions of the image, by using different values for the
        # contrast limit, the upper limit should be determined
        # and used carefully. From one image to another, this can be variable.
        cl = self.clip_limit
        clahe = cv2.createCLAHE(clipLimit=cl, tileGridSize=(2, 2))
        img_lab[:, :, 0] = clahe.apply(img_lab[:, :, 0])

        img_lab[:, :, 0] = cv2.equalizeHist(img_lab[:, :, 0])

        # convert image back from LAB to RGB
        equalized_image = cv2.cvtColor(img_lab, cv2.COLOR_LAB2RGB)

        with rasterio.open(os.path.join(
                path + f'/{filename}.tif'), 'w', **profile) as dst:
            dst.write(np.moveaxis(equalized_image, -1, 0))

        hist_size = 2 ** 8 - 1
        hist_range = (1, 2 ** 8 - 1)

        h1 = cv2.calcHist([image], [0], None, [hist_size], hist_range)
        h2 = cv2.calcHist([equalized_image],
                          [0], None, [hist_size], hist_range)

        self.__plot([h1, h2], path, filename)

        self.output = _load_image(os.path.join(path + f'/{filename}.tif'))
        self.commit.deferred()

    def _show_histogram(self):

        ds, filename, path = self.__read_image(self.combobox_node)
        image = ds.read()
        image = np.moveaxis(image, 0, -1)

        hist_size = 2 ** 8 - 1
        hist_range = [1, 2 ** 8 - 1]

        hist = cv2.calcHist([image], [0], None, [hist_size], hist_range)
        self.__plot([hist], path, filename)

        self.combobox_reference.clear()
        self.combobox_reference.addItems(
            [node.name for node in self.input_nodes])

        self.combobox_to_match.clear()
        self.combobox_to_match.addItems(
            [node.name for node in self.input_nodes])

    def __read_image(self, combobox):
        idx_node = combobox.currentIndex()
        image_node = self.input_nodes[idx_node]

        path = image_node.path.path
        name = os.path.basename(path)
        filename = os.path.splitext(name)[0]

        path_r = os.path.join(os.path.dirname(path), 'histogram')

        if not os.path.exists(path_r):
            os.mkdir(os.path.join(os.path.dirname(path), 'histogram'))

        ds = image_node.get_impl(rasterio.DatasetReader)

        return ds, filename, path_r

    def __plot(self, hist, path, filename):

        plt.figure()
        for h in hist:
            try:
                plt.plot(h)
            except ValueError:
                plt.plot(h[0])
        plt.xlim([0, 256])
        # plt.show()
        plt.savefig(os.path.join(path + f'/{filename}.png'))

        pic = QGraphicsPixmapItem()
        pic.setPixmap(QPixmap(os.path.join(path + f'/{filename}.png')))
        self.box_scene.addItem(pic)

    @Inputs.data
    def set_data(self, index, node):
        try:
            self.input_nodes[index] = node
        except IndexError:
            self.input_nodes.append(node)

        self.combobox_node.clear()
        self.combobox_node.addItems([node.name for node in self.input_nodes])

    @Inputs.data.insert
    def insert_data(self, index, node: DrbImageBaseNode):

        if index == 0:
            self.input_nodes = []

        self.input_nodes.append(node)
        self.combobox_node.clear()
        self.combobox_node.addItems([node.name for node in self.input_nodes])

    @Inputs.data.remove
    def remove_data(self, index):
        self.input_nodes.pop(index)
        self.combobox_node.clear()
        self.combobox_node.addItems([node.name for node in self.input_nodes])

        if not self.input_nodes:
            self.combobox_node.clear()

    @gui.deferred
    def commit(self):
        """
        Commits the result the next widget in the line.
        """
        self.Outputs.output.send(self.output)


def _load_image(file):
    """
    Loads and create a DrbImageBaseNode from local file.
    Parameters:
        file (str): path to local file
    Returns:
        DrbImageBaseNode: a Drb Node of the TC Image
    """
    base_node = DrbFileFactory().create(file)
    image_node = DrbImageFactory().create(base_node)

    return image_node


if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview
    WidgetPreview(OWHistogram).run()



import os
import rasterio
from rasterio.crs import CRS
from rasterio.warp import calculate_default_transform, reproject, Resampling
from Orange.widgets import widget, gui, settings
from Orange.widgets.utils.signals import Input, Output
from drb.drivers.file import DrbFileFactory
from drb.drivers.image import DrbImageFactory, DrbImageBaseNode


class OWReprojectImage(widget.OWWidget):
    name = "EO Reproject Image"
    description = "Projects an image into a different CRS"
    icon = "icons/projection.svg"
    priority = 40

    epsg = []
    output_epsg = settings.Setting(4326)
    node = None

    want_main_area = False
    graph_name = "thumbnailView"
    auto_commit = settings.Setting(True)

    class Inputs:
        data = Input("DrbImageBaseNode",
                     DrbImageBaseNode, auto_summary=False)

    class Outputs:
        output = Output("DrbImageBaseNode",
                        DrbImageBaseNode, auto_summary=False)

    def __init__(self):
        super().__init__()
        self.output = None

        self.combobox = gui.comboBox(
            self.controlArea, self, "", items=self.epsg,
            box="Selected Input CRS:",
            contentsLength=12,
        )

        box1 = gui.vBox(self.controlArea, 'Select Output CRS:')
        gui.spin(box1, self, 'output_epsg', 1, 100000,
                 label='                       EPSG:')

        gui.rubber(self.controlArea)

        box2 = gui.widgetBox(self.controlArea)
        self.button = gui.button(
            box2, self, "Reproject Image", callback=self.reproject_image
        )

        gui.rubber(self.controlArea)
        gui.auto_commit(self.buttonsArea, self,
                        "auto_commit", "Send", box=False)

    def reproject_image(self):

        path = self.node.path.path
        output_crs = CRS.from_epsg(self.output_epsg)

        image_node = self.node
        name = os.path.basename(path)
        filename = os.path.splitext(name)[0]
        filename = filename.split('_')[0] + f'_{output_crs}'

        src = image_node.get_impl(rasterio.DatasetReader)

        transform, width, height = calculate_default_transform(
            src.crs, output_crs, src.width, src.height, *src.bounds)
        kwargs = src.meta.copy()
        kwargs.update({

            'driver': 'GTiff',
            'crs': output_crs,
            'transform': transform,
            'width': width,
            'height': height
        })

        with rasterio.open(os.path.join(os.path.dirname(
                path) + f'/{filename}.tif'), 'w', **kwargs) as dst:
            for i in range(1, src.count + 1):
                reproject(
                    source=rasterio.band(src, i),
                    destination=rasterio.band(dst, i),
                    src_transform=src.transform,
                    src_crs=src.crs,
                    dst_transform=transform,
                    dst_crs=output_crs,
                    resampling=Resampling.nearest)

        self.output = _load_image(os.path.join(
            os.path.dirname(path) + f'/{filename}.tif'))
        self.commit.deferred()

    @Inputs.data
    def set_data(self, node):

        self.node = node
        if self.node:
            ds = self.node.get_impl(rasterio.DatasetReader)
            self.epsg = [CRS.to_epsg(ds.crs)]

            self.combobox.clear()  # delete all items from comboBox
            self.combobox.addItems([f'EPSG: {epsg}' for epsg in self.epsg])

    @gui.deferred
    def commit(self):
        """
        Commits the result the next widget in the line.
        """
        self.Outputs.output.send(self.output)


def _load_image(file):
    """
    Loads and create a DrbImageBaseNode from local file.
    Parameters:
        file (str): path to local file
    Returns:
        DrbImageBaseNode: a Drb Node of the TC Image
    """
    base_node = DrbFileFactory().create(file)
    image_node = DrbImageFactory().create(base_node)

    return image_node


if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview
    WidgetPreview(OWReprojectImage).run()



import re
from drb.core.predicate import Predicate


class RegexNamePredicate(Predicate):
    def __init__(self, regex: str):
        self._regex = re.compile(regex)

    def matches(self, node):
        return self._regex.match(node.name) is not None



import os
import shapefile
import rasterio
from rasterio.mask import mask
from rasterio.crs import CRS
from rasterio.warp import transform_bounds, calculate_default_transform, \
    reproject, Resampling
from Orange.widgets import widget, gui, settings
from Orange.widgets.utils.signals import Input, Output
from PyQt5.QtWidgets import QAction, QFileDialog, QPushButton
from drb.core import DrbNode
from drb.drivers.file import DrbFileFactory
from drb.drivers.image import DrbImageFactory, DrbImageBaseNode
from .utils import RegexNamePredicate


class OWNUTSShape(widget.OWWidget):
    name = "EO NUTS Shape"
    description = "Applies on the given Sentinel 2 True Color image, " \
                  "a given mask corresponding to NUTS shape"
    icon = "icons/nuts.svg"
    priority = 20

    shape_recs = []
    nuts_available = []
    nodes = []

    want_main_area = False
    graph_name = "thumbnailView"
    auto_commit = settings.Setting(True)
    nuts_url = "https://gisco-services.ec.europa.eu/distribution/v2/nuts/" \
               "shp/NUTS_RG_20M_2021_4326.shp.zip"

    class Inputs:
        data = Input("DrbNode", DrbNode, auto_summary=False)

    class Outputs:
        output = Output("DrbImageBaseNode",
                        DrbImageBaseNode, auto_summary=False)

    def __init__(self):
        super().__init__()
        self.output = None

        self.combobox_node = gui.comboBox(
            self.controlArea, self, "", items=self.nodes,
            box="Select Node:",
            contentsLength=12,
        )
        self.combobox_node.activated.connect(self.__add_nuts_options)
        gui.rubber(self.controlArea)

        self.box = gui.widgetBox(self.controlArea, "Select NUTS Region:")
        self.combobox_nuts = gui.comboBox(
            self.box, self, "", items=self.nuts_available,
            contentsLength=12,
        )

        self.button = gui.button(
            self.box, self, "Apply NUTS Shape",
            callback=self.apply_nuts_shape
        )

        gui.rubber(self.controlArea)
        gui.auto_commit(self.buttonsArea, self,
                        "auto_commit", "Send", box=False)

    def apply_nuts_shape(self):

        if self.nodes:
            idx_nuts = self.combobox_nuts.currentIndex()

            shp = shapefile.Reader(self.nuts_url)
            shape_rec = self.shape_recs[idx_nuts]
            polygon = shape_rec[0]
            filename = shape_rec[1]
            shape = shp.shape(polygon)

            idx_node = self.combobox_node.currentIndex()
            node = self.nodes[idx_node]

            path = os.path.splitext(node.path.path)[0]

            if not os.path.exists(path):
                os.mkdir(path)

            if isinstance(node, DrbImageBaseNode):
                image_node = node
                ds = image_node.get_impl(rasterio.DatasetReader)

            else:
                bands = node[0]['GRANULE'][0]['IMG_DATA']['R60m']
                image_node = bands[RegexNamePredicate(".*TCI*")]
                ds = image_node[0].get_impl(rasterio.DatasetReader)

            if ds.crs == CRS.from_epsg(4326):
                image = ds
            else:
                image = self._reproject_image(ds, path)

            out_image, out_transform = mask(image, [shape], crop=True)

            out_meta = ({"height": out_image.shape[1],
                         "width": out_image.shape[2],
                         "transform": out_transform,
                         "driver": 'GTiff',
                         "dtype": 'uint8',
                         "count": 3,
                         "crs": CRS.from_epsg(4326)
                         })
            with rasterio.open(os.path.join(
                    path + f'/{filename}.tif'), "w", **out_meta) as dst:
                dst.write(out_image)

            self.output = _load_image(os.path.join(path + f'/{filename}.tif'))
            self.commit.deferred()

    @staticmethod
    def _reproject_image(ds, path):
        """
        Re-projects an image from a CRS to the (4326) CRS
        corresponding to the NUTS shapefile's projection.
        """
        crs_4326 = CRS.from_epsg(4326)

        src = ds
        transform, width, height = calculate_default_transform(
            src.crs, crs_4326, src.width, src.height, *src.bounds)
        kwargs = src.meta.copy()
        kwargs.update({
            'crs': crs_4326,
            'transform': transform,
            'width': width,
            'height': height,
            'driver': 'GTiff'
        })

        with rasterio.open(os.path.join(
                path, 'TCI_4326.tif'), 'w', **kwargs) as dst:

            for i in range(1, src.count + 1):
                reproject(
                    source=rasterio.band(src, i),
                    destination=rasterio.band(dst, i),
                    src_transform=src.transform,
                    src_crs=src.crs,
                    dst_transform=transform,
                    dst_crs=crs_4326,
                    resampling=Resampling.nearest)

        reprojected_image = rasterio.open(os.path.join(
            path + '/TCI_4326.tif'))
        return reprojected_image

    def __add_nuts_options(self):
        """
        Adds a list of all available NUTS regions to
        the combobox for user to choose.
        """
        if self.nodes:
            idx = self.combobox_node.currentIndex()
            node = self.nodes[idx]

            if isinstance(node, DrbImageBaseNode):
                image_node = node
                ds = image_node.get_impl(rasterio.DatasetReader)

            else:
                bands = node[0]['GRANULE'][0]['IMG_DATA']['R60m']
                image_node = bands[RegexNamePredicate(".*TCI*")]
                ds = image_node[0].get_impl(rasterio.DatasetReader)

            if ds.crs == CRS.from_epsg(4326):
                bounds = ds.bounds
            else:
                bounds = transform_bounds(
                    ds.crs, CRS.from_epsg(4326), ds.bounds.left,
                    ds.bounds.bottom, ds.bounds.right, ds.bounds.top)
            shp = shapefile.Reader(self.nuts_url)

            self.shape_recs = [(shapeRec.record.oid,
                                f'NUTS {shapeRec.record[1]}'
                                f' - {shapeRec.record[3]}')
                               for shapeRec in
                               shp.iterShapeRecords(bbox=bounds)]

            self.nuts_available = [shapeRec[1] for shapeRec in self.shape_recs]

            self.combobox_nuts.clear()
            self.combobox_nuts.addItems(self.nuts_available)

    @Inputs.data
    def set_data(self, nodes):
        if not isinstance(nodes, list):
            self.nodes = [nodes]
        elif isinstance(nodes, list):
            self.nodes = nodes

        self.combobox_node.clear()
        self.combobox_node.addItems([node.name for node in self.nodes])

    @gui.deferred
    def commit(self):
        """
        Commits the result the next widget in the line.
        """
        self.Outputs.output.send(self.output)


def _load_image(file):
    """
    Loads and create a DrbImageBaseNode from local file.
    Parameters:
        file (str): path to local file
    Returns:
        DrbImageBaseNode: a Drb Node of the TC Image
    """
    base_node = DrbFileFactory().create(file)
    image_node = DrbImageFactory().create(base_node)

    return image_node


if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview
    WidgetPreview(OWNUTSShape).run()



import io
from Orange.widgets import widget, gui, settings
from Orange.widgets.utils.signals import Output
from PyQt5.QtCore import Qt, QDate
from PyQt5.QtGui import QPalette, QColor, QTextCharFormat
from PyQt5.QtWidgets import QAction, QPushButton, \
    QFileDialog, QCalendarWidget, QHBoxLayout
from drb.core import DrbNode
from drb.utils.keyringconnection import kr_add
from drb.drivers.odata import ODataQueryPredicate, OdataFactory
import drb.topics.resolver as resolver
import datetime


class CalendarStateMachine:
    def __init__(self, calendar):
        self.calendar = calendar
        self.palette = self.calendar.palette()
        self.first_date = None
        self.second_date = None
        self.calendar.clicked[QDate].connect(self.handle_clicked)

    def set_selected_style(self):
        self.palette.setColor(QPalette.Highlight, QColor("#007acc"))
        self.palette.setColor(QPalette.HighlightedText, Qt.white)
        self.calendar.setPalette(self.palette)

        if self.first_date is not None and self.second_date is not None:
            fmt = QTextCharFormat()
            fmt.setBackground(self.palette.color(QPalette.Highlight))
            fmt.setForeground(self.palette.color(QPalette.HighlightedText))
            start_date = self.first_date
            while start_date <= self.second_date:
                self.calendar.setDateTextFormat(start_date, fmt)
                start_date = start_date.addDays(1)

    def reset_calendar_style(self):
        # Reset the calendar's palette to the default palette
        self.calendar.setPalette(self.calendar.style().standardPalette())

        # Remove any custom date text formats that were applied
        if self.first_date is not None and self.second_date is not None:
            fmt = QTextCharFormat()
            start_date = self.first_date
            while start_date <= self.second_date:
                self.calendar.setDateTextFormat(start_date, fmt)
                start_date = start_date.addDays(1)

    def handle_clicked(self, date):
        if self.first_date is None:
            self.first_date = date
            self.set_selected_style()
        elif self.second_date is None:
            if self.first_date < date:
                self.second_date = date
                self.set_selected_style()
            elif self.first_date == date:
                self.reset_calendar_style()
                self.first_date = None

        else:
            if self.first_date <= date <= self.second_date:
                self.reset_calendar_style()

                self.first_date = None
                self.second_date = None
            elif self.first_date > date:
                self.first_date = date
                self.set_selected_style()
            elif self.second_date < date:
                self.second_date = date
                self.set_selected_style()


class OWDataCatalog(widget.OWWidget):
    name = "EO Data Catalog"
    description = "Access and retrieve data from " \
                  "different services using Odata protocol"
    icon = "icons/catalog.svg"
    priority = 10

    services = ["CSC Service", "DHuS Service", "Dias Service"]
    missions = ["S1", "S2", "S3",
                "S5p", "EN1", "LC08"]
    platforms = []
    product_types = []
    sensors = []
    nodes = []
    output_nodes = []

    cloud_cover_value = 20
    latitude = 48.8
    longitude = 2.4

    modality = Qt.ApplicationModal
    current_path = os.getcwd()
    recent_paths = settings.Setting([])

    want_main_area = False
    graph_name = "thumbnailView"
    auto_commit = settings.Setting(True)

    class Outputs:
        output = Output("Data", DrbNode, auto_summary=False)

    def __init__(self):
        super().__init__()
        self.data = None
        self.username = None
        self.password = None
        self.service = None
        self.service_node = None
        self.output = None

        # GUI Commands
        self.combobox_service = gui.comboBox(
            self.controlArea, self, "", items=self.services,
            label="Choose Service: ",
            orientation=Qt.Horizontal,
            contentsLength=8,
        )

        gui.rubber(self.controlArea)

        self.authentication_box = gui.widgetBox(
            self.controlArea, "Authentication: ")
        gui.rubber(self.controlArea)

        self.username_line = gui.lineEdit(
            self.authentication_box, self, "",
            label="Enter username:",
            orientation=Qt.Horizontal,
            controlWidth=400,
        )
        self.password_line = gui.lineEdit(
            self.authentication_box, self, "",
            label="Enter password:",
            orientation=Qt.Horizontal,
            controlWidth=400,
        )

        self.query_button = gui.button(
            self.authentication_box, self, "Query Service",
            autoDefault=False,
            callback=self.query_service
        )

        self.calendar = QCalendarWidget()
        self.calendar.setGeometry(0, 0, 100, 80)
        self.statemachine = CalendarStateMachine(self.calendar)
        self.controlArea.layout().addWidget(self.calendar)

        self.predicate_box = gui.widgetBox(self.controlArea, " ")
        gui.rubber(self.controlArea)

        self.combobox_mission = gui.comboBox(
            self.predicate_box, self, "", items=self.missions,
            label="Select Mission:",
            orientation=Qt.Horizontal,
            contentsLength=12,
        )
        self.combobox_mission.activated.connect(self.choose_mission)

        self.combobox_platform = gui.comboBox(
            self.predicate_box, self, "", items=self.platforms,
            label="Select Platform:",
            orientation=Qt.Horizontal,
            contentsLength=12,
        )

        self.combobox_product_type = gui.comboBox(
            self.predicate_box, self, "", items=self.product_types,
            label="Select Product Type:",
            orientation=Qt.Horizontal,
            contentsLength=12,
        )

        self.combobox_sensor = gui.comboBox(
            self.predicate_box, self, "", items=self.sensors,
            label="Select Sensor:",
            orientation=Qt.Horizontal,
            contentsLength=12,
        )

        self.slider_cc_value = gui.hSlider(
            self.predicate_box, self, "cloud_cover_value",
            label="Cloud Cover Value %:", minValue=0, maxValue=100, step=1,
            width=400,
            createLabel=True
        )

        self.lat_spinbox = gui.doubleSpin(
            self.predicate_box, self, "latitude", -90, 90,
            label="Latitude:",
            orientation=Qt.Horizontal,
            decimals=2
        )
        self.lon_spinbox = gui.doubleSpin(
            self.predicate_box, self, "longitude", -180, 180,
            label="Longitude:",
            orientation=Qt.Horizontal,
            decimals=2
        )

        self.filter_button = gui.button(
            self.predicate_box, self, "Apply Predicate",
            callback=self.apply_filter
        )

        self.selection_box = gui.widgetBox(
            self.controlArea, "Select Node(s):")
        gui.rubber(self.controlArea)

        self.combobox_node_selection = gui.comboBox(
            self.selection_box, self, "", items=self.nodes,
            contentsLength=12,
        )
        self.combobox_node_selection.activated.connect(
            self.send_node)

        self.selection_button = gui.button(
            self.selection_box, self, "Select All",
            callback=self.send_all
        )

        browse_action = QAction(
            self,
            text="\N{HORIZONTAL ELLIPSIS}",
        )

        browse_action.triggered.connect(self.__open_dialog)
        browse_button = QPushButton(
            browse_action.iconText(),
            clicked=browse_action.trigger
        )

        vbox = gui.vBox(self.controlArea, box="Select Directory to:")
        hbox = gui.hBox(vbox)
        hbox.layout().addWidget(browse_button)

        self.download_button = gui.button(
            vbox, self, "Download Selection",
            callback=self.download
        )

        gui.rubber(self.controlArea)
        gui.auto_commit(self.buttonsArea, self,
                        "auto_commit", "Send", box=False)

    def query_service(self):

        self.choose_service()

        self.username = self.username_line.text()
        self.username_line.clear()
        self.password = self.password_line.text()
        self.password_line.clear()

        kr_add(service=self.service,
               username=self.username,
               password=self.password)

        self.service_node = OdataFactory().create(self.service)
        gui.label(
            self.authentication_box, self,
            f'{len(self.service_node.children)} results found')

    def choose_service(self):
        idx = self.combobox_service.currentIndex()

        # CSC
        if idx == 0:
            self.service = 'https://demo1.databridge.gael.fr/gss-catalogue'

        # DHuS
        elif idx == 1:
            self.service = 'https://apihub.copernicus.eu/apihub/odata/v2'

            # DIAS
        elif idx == 2:
            self.service = 'https://catalogue.onda-dias.eu/dias-catalogue'

    def choose_mission(self):
        idx = self.combobox_mission.currentIndex()

        # Sentinel1
        if idx == 0:

            self.platforms = ["", "A", "B"]
            self.combobox_platform.clear()
            self.combobox_platform.addItems([p for p in self.platforms])

            self.product_types = ["", "_SLC_", "_GRD_",
                                  "_OCN_", "_RAW_"]
            self.combobox_product_type.clear()
            self.combobox_product_type.addItems(
                [pt for pt in self.product_types])

            self.sensors = ["", "SW", "IW", "EW", "WV"]
            self.combobox_sensor.clear()
            self.combobox_sensor.addItems([s for s in self.sensors])

        # Sentinel2
        elif idx == 1:

            self.platforms = ["", "A", "B"]
            self.combobox_platform.clear()
            self.combobox_platform.addItems([p for p in self.platforms])

            self.product_types = ["", "_MSIL1C_", "_MSIL2A_"]
            self.combobox_product_type.clear()
            self.combobox_product_type.addItems(
                [pt for pt in self.product_types])
            self.combobox_sensor.clear()

        # Sentinel3
        elif idx == 2:

            self.platforms = ["", "A", "B"]
            self.combobox_platform.clear()
            self.combobox_platform.addItems(
                [p for p in self.platforms])

            self.product_types = ["", "_SR_1_SRA___", "_SR_1_SRA_A_",
                                  "_SR_1_SRA_BS",
                                  "_SR_2_LAN___", "_OL_1_EFR___",
                                  "_OL_1_ERR___",
                                  "_SL_1_RBT___", "_OL_2_LFR___",
                                  "_OL_2_LRR___",
                                  "_SL_2_LST___", "_OL_2_WFR___",
                                  "_OL_2_WRR___",
                                  "_SL_2_WST___", "_SR_2_WAT___",
                                  "_SY_2_SYN___",
                                  "_SY_2_V10___", "_SY_2_VG1___",
                                  "_SY_2_VGP___"]
            self.combobox_product_type.clear()
            self.combobox_product_type.addItems(
                [pt for pt in self.product_types])
            self.combobox_sensor.clear()

        # Sentinel5p
        elif idx == 3:

            self.platforms = ["", "_OFFL_", "_NRTI_"]
            self.combobox_platform.clear()
            self.combobox_platform.addItems([p for p in self.platforms])

            self.product_types = ["", "L1B_IR_SIR", "L1B_IR_UVN", "L1B_RA_BD1",
                                  "L1B_RA_BD2", "L1B_RA_BD3", "L1B_RA_BD4",
                                  "L1B_RA_BD5", "L1B_RA_BD6", "L1B_RA_BD7",
                                  "L1B_RA_BD8", "L2__AER_AI", "L2__CH4___",
                                  "L2__CLOUD_", "L2__SO2___", "L2__CO____",
                                  "L2__HCHO__", "L2__NO2___", "L2__O3____",
                                  "L2__O3_TCL_"]
            self.combobox_product_type.clear()
            self.combobox_product_type.addItems(
                [pt for pt in self.product_types])
            self.combobox_sensor.clear()

        # EnviSat
        elif idx == 4:

            self.platforms = ["", "_SPDK_", "_SPDE_"]
            self.combobox_platform.clear()
            self.combobox_platform.addItems([p for p in self.platforms])

            self.product_types = ["", "ASA_IM__OP", "ASA_WS__OP"]
            self.combobox_product_type.clear()
            self.combobox_product_type.addItems(
                [pt for pt in self.product_types])
            self.combobox_sensor.clear()

        # Landsat8
        elif idx == 5:

            self.platforms = [""]
            self.combobox_platform.clear()
            self.combobox_platform.addItems([p for p in self.platforms])

            self.product_types = ["", "_L1TP_"]
            self.combobox_product_type.clear()
            self.combobox_product_type.addItems(
                [pt for pt in self.product_types])
            self.combobox_sensor.clear()

    def apply_filter(self):

        idx_service = self.combobox_service.currentIndex()
        idx_mission = self.combobox_mission.currentIndex()
        mission = self.missions[idx_mission]

        idx_platform = self.combobox_platform.currentIndex()
        platform = self.platforms[idx_platform]

        idx_type = self.combobox_product_type.currentIndex()
        pt = self.product_types[idx_type]

        if idx_mission == 0:
            idx_sensor = self.combobox_sensor.currentIndex()
            sensor = self.sensors[idx_sensor]
        else:
            sensor = ""

        predicate = f"startswith("f"Name, '{mission}{platform}{sensor}{pt}')"

        # cloud coverage
        if idx_mission == 1 or idx_mission == 5:
            cc = self.cloud_cover_value
            predicate = predicate + \
                f" and Attributes/OData.CSC.DoubleAttribute/any(" \
                f"att:att/Name eq 'cloudCover' and " \
                f"att/OData.CSC.DoubleAttribute/Value lt {cc})"

        long_min = self.longitude - 0.5
        long_max = self.longitude + 0.5
        lat_min = self.latitude - 0.5
        lat_max = self.latitude + 0.5

        WTK_area = f"Polygon(({long_min} {lat_max}," \
                   f"{long_max} {lat_max},{long_max} {lat_min}," \
                   f"{long_min} {lat_min},{long_min} {lat_max}))"
        predicate = predicate + f" and OData.CSC.Intersects(" \
                                f"location=Footprint," \
                                f"area=geography'SRID=4326;" \
                                f"{WTK_area}')"

        start_date = self.statemachine.first_date
        end_date = self.statemachine.second_date

        if start_date is not None:
            if end_date is None:
                date = datetime.datetime(start_date.year(),
                                         start_date.month(), start_date.day())
                iso_date = date.isoformat() + "Z"
                predicate = predicate + f" and ContentDate/Start gt {iso_date}"
            else:
                qdate = datetime.datetime(start_date.year(),
                                          start_date.month(), start_date.day())
                iso_start_date = qdate.isoformat() + "Z"
                zdate = datetime.datetime(end_date.year(),
                                          end_date.month(), end_date.day())
                iso_end_date = zdate.isoformat() + "Z"

                predicate = predicate + \
                    f" and ContentDate/Start gt {iso_start_date} " \
                    f"and ContentDate/Start lt {iso_end_date}"

        if idx_service == 2:
            filtered_children = ODataQueryPredicate(search=predicate)
        else:
            filtered_children = ODataQueryPredicate(filter=predicate)

        children = self.service_node[filtered_children]
        gui.label(
            self.predicate_box, self, f'{len(children)} results found')

        self.nodes = []

        if idx_mission == 1 or idx_mission == 2:
            if len(children) < 100:
                for i in range(len(children)):
                    self.nodes.append(children[i].name)

                self.combobox_node_selection.clear()
                self.combobox_node_selection.addItems(
                    [node for node in self.nodes])

    def send_node(self):

        idx = self.combobox_node_selection.currentIndex()
        product_name = self.nodes[idx]
        product_node = self.service_node[product_name]
        self.output = product_node
        self.commit.deferred()

    def send_all(self):

        for product_node in self.service_node.children:
            self.output_nodes.append(product_node)

        self.output = self.output_nodes
        self.output_nodes = []
        self.commit.deferred()

    def download(self):

        idx = self.combobox_node_selection.currentIndex()
        product_name = self.nodes[idx]
        product_node = self.service_node[product_name]

        with product_node.get_impl(io.BytesIO) as stream:
            with open(os.path.join(
                    self.current_path + f'/{product_name}'), 'wb') as f:
                f.write(stream.read())
            f.close()

    @gui.deferred
    def commit(self):
        """
        Commits the result the next widget in the line.
        """
        self.Outputs.output.send(self.output)

    def __open_dialog(self):
        start_dir = os.path.expanduser("~/")
        if self.recent_paths:
            start_dir = os.path.dirname(self.recent_paths[0].abspath)

        if OWDataCatalog.modality == Qt.WindowModal:
            dlg = QFileDialog(
                self, "Select Directory to Download to", start_dir,
                acceptMode=QFileDialog.AcceptOpen,
                modal=True,
            )
            dlg.setFileMode(QFileDialog.Directory)
            dlg.setOption(QFileDialog.ShowDirsOnly)
            dlg.setDirectory(start_dir)
            dlg.setAttribute(Qt.WA_DeleteOnClose)

            @dlg.accepted.connect
            def on_accepted():
                path_dir = dlg.selectedFiles()
                if dir_path:
                    self.setCurrentPath(path_dir[0])
                    self.start()
            dlg.open()
        else:
            dir_path = QFileDialog.getExistingDirectory(
                self, "Select Directory to Download to", start_dir
            )
            if dir_path:
                self.current_path = dir_path

    def handleClicked(self, qDate):
        if self.calendar.selectedDate() == self.startDate:
            # If the selected date is the start date, update the end date
            self.endDate = qDate
        elif self.calendar.selectedDate() == self.endDate:
            # If the selected date is the end date, update the start date
            self.startDate = qDate
        else:
            # If the selected date is not the start or end date,
            # reset the range
            self.startDate = qDate
            self.endDate = qDate

        # Select the range of dates
        self.calendar.setMaximumDate(self.startDate)
        self.calendar.setMaximumDate(self.endDate)
        print(self.startDate)
        print(self.endDate)


if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview
    WidgetPreview(OWDataCatalog).run()



from typing import List, Optional, Iterable
from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import Input, Output, MultiInput
from PyQt5.QtCore import QSize
from PyQt5.QtGui import QPixmap, QIcon
from PyQt5.QtWidgets import QWidget, QListWidget, QListWidgetItem, QListView
from drb.drivers.file import DrbFileFactory
from drb.drivers.image import DrbImageFactory, DrbImageBaseNode


class IconView(QListWidget):
    """
    A list view (in QListView.IconMode).
    """

    def __init__(
            self, parent: Optional[QWidget] = None,
            icon_size=QSize(80, 80),
            word_wrap=True,
            **kwargs
    ) -> None:
        super().__init__(parent, wordWrap=word_wrap, **kwargs)
        self.setViewMode(QListView.IconMode)
        self.setEditTriggers(QListView.NoEditTriggers)
        self.setMovement(QListView.Static)
        self.setSelectionMode(QListView.ExtendedSelection)
        self.setIconSize(icon_size)


class OWViewImage(widget.OWWidget):
    name = "EO View Image"
    description = "Views image(s), also can be used to save an" \
                  " image in the directory structure"
    icon = "icons/view.svg"
    priority = 80

    input_nodes = []
    selection = None

    want_main_area = True
    graph_name = "thumbnailView"
    image_size = settings.Setting(100)
    auto_commit = settings.Setting(True)

    class Inputs:
        data = MultiInput("List[DrbImageBaseNode]",
                          DrbImageBaseNode, auto_summary=False)

    class Outputs:
        output = Output("Selected[DrbImageBaseNode]",
                        DrbImageBaseNode, auto_summary=False)

    def __init__(self):
        super().__init__()
        self.nodes = []
        self.output = None

        gui.hSlider(
            self.controlArea, self, "image_size",
            box="Image Size", minValue=32, maxValue=1024, step=16,
            callback=self._update_size,
            createLabel=False
        )
        gui.rubber(self.controlArea)
        gui.auto_commit(self.buttonsArea, self,
                        "auto_commit", "Send", box=False)

        self.thumbnailView = IconView(
            resizeMode=IconView.Adjust,
            iconSize=QSize(self.image_size, self.image_size),
        )
        self.thumbnailView.itemClicked.connect(self._change_selection)

        self.main_scene = self.mainArea.layout()
        self.mainArea.setMinimumWidth(200)

    def view_image(self):

        self.thumbnailView.clear()
        if self.input_nodes:
            for image_node in self.input_nodes:
                print(image_node)
                path = image_node.path.path

                item = QListWidgetItem()
                icon = QIcon()
                icon.addPixmap(QPixmap(path))
                item.setIcon(icon)
                self.thumbnailView.addItem(item)

            self.main_scene.addWidget(self.thumbnailView)

    def _update_size(self):
        self.thumbnailView.setIconSize(QSize(self.image_size, self.image_size))

    @Slot()
    def _change_selection(self):

        self.selection = self.thumbnailView.currentRow()
        self.commit.deferred()

    @Inputs.data
    def set_data(self, index, image: DrbImageBaseNode):
        try:
            self.input_nodes[index] = image
        except IndexError:
            self.input_nodes.append(image)

        self.view_image()

    @Inputs.data.insert
    def insert_data(self, index, image: DrbImageBaseNode):

        if index == 0:
            self.input_nodes = [image]

        else:
            self.input_nodes.append(image)

        self.view_image()

    @Inputs.data.remove
    def remove_data(self, index):
        self.input_nodes.pop(index)

        if not self.input_nodes:
            self.combobox_image.clear()

        self.view_image()

    @gui.deferred
    def commit(self):
        """
        Commits the result the next widget in the line.
        """

        if self.input_nodes:
            if self.selection is not None:
                self.output = self.input_nodes[self.selection]
                self.Outputs.output.send(self.output)


if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview
    WidgetPreview(OWViewImage).run()



import numpy as np
import os
import rasterio
from Orange.widgets import widget, gui
from Orange.widgets.utils.signals import Input, Output
from Orange.widgets import settings
from PyQt5.QtCore import Qt
from drb.core import DrbNode
from drb.drivers.file import DrbFileFactory
from drb.drivers.image import DrbImageFactory, DrbImageBaseNode
from .utils import RegexNamePredicate


class OWCloudMask(widget.OWWidget):
    name = "EO Cloud Mask"
    description = "Applies on the given Sentinel 2 True Color image, " \
                  "a given clouds, shadows, water or snow-ice mask"
    icon = "icons/cloud.svg"
    priority = 50

    mask_options = ["Clouds", 'Clouds and Shadow', "Water", "Snow--Ice"]
    nodes = []

    want_main_area = False
    graph_name = "thumbnailView"
    auto_commit = settings.Setting(True)

    class Inputs:
        data = Input("DrbNode", DrbNode, auto_summary=False)

    class Outputs:
        output = Output("DrbImageBaseNode",
                        DrbImageBaseNode, auto_summary=False)

    def __init__(self):
        super().__init__()
        self.output = None

        self.node_box = gui.widgetBox(self.controlArea, "Select Node: ")

        self.combobox_node = gui.comboBox(
            self.node_box, self, "",
            contentsLength=12,
        )

        gui.rubber(self.controlArea)
        self.mask_box = gui.widgetBox(self.controlArea, "Select Mask: ")

        self.combobox_masks = gui.comboBox(
            self.mask_box, self, "", items=self.mask_options,
            contentsLength=12,
        )

        self.button = gui.button(
            self.mask_box, self, "Apply Mask",
            callback=self.apply_mask
        )

        gui.rubber(self.controlArea)
        gui.auto_commit(self.buttonsArea, self,
                        "auto_commit", "Send", box=False)

    def apply_mask(self):

        if self.nodes:
            idx_node = self.combobox_node.currentIndex()
            node = self.nodes[idx_node]

            path = os.path.splitext(node.path.path)[0]

            if not os.path.exists(path):
                os.mkdir(path)

            bands = node[0]['GRANULE'][0]['IMG_DATA']['R60m']

            mask_node = bands[RegexNamePredicate(".*SCL*")]
            ds_mask = mask_node[0].get_impl(rasterio.DatasetReader)
            mask = ds_mask.read()

            image_node = bands[RegexNamePredicate(".*TCI*")]
            ds_image = image_node[0].get_impl(rasterio.DatasetReader)
            image = ds_image.read()
            profile = ds_image.meta.copy()

            idx = self.combobox_masks.currentIndex()
            mask_name = self.mask_options[idx]

            # preparing mask based on user selection
            if idx == 0:  # Clouds

                mask1 = (mask != 8).astype(int)
                mask2 = (mask != 9).astype(int)
                mask3 = (mask != 10).astype(int)

                prep_mask = np.logical_and(mask1, mask2).astype(int)
                prep_mask = np.logical_and(prep_mask, mask3).astype(int)

            if idx == 1:  # Shadows

                mask1 = (mask != 8).astype(int)
                mask2 = (mask != 9).astype(int)
                mask3 = (mask != 3).astype(int)
                mask4 = (mask != 10).astype(int)

                prep_mask = np.logical_and(mask1, mask2).astype(int)
                prep_mask = np.logical_and(prep_mask, mask3).astype(int)
                prep_mask = np.logical_and(prep_mask, mask4).astype(int)

            if idx == 2:  # Water
                prep_mask = (mask != 6).astype(int)

            if idx == 3:  # Snow/Ice
                prep_mask = (mask != 11).astype(int)

            prep_mask = np.moveaxis(prep_mask, 0, -1)
            image = np.moveaxis(image, 0, -1)
            result_image = image * prep_mask
            result_image = np.moveaxis(result_image, -1, 0)

            profile.update({
                'driver': 'GTiff',
            })

            with rasterio.open(os.path.join(
                    path + f'/TCI-without-{mask_name}.tif'),
                    'w', **profile) as dst:
                dst.write(result_image)

            self.output = _load_image(os.path.join(
                path + f'/TCI-without-{mask_name}.tif'))
            self.commit.deferred()

    @Inputs.data
    def set_data(self, nodes):
        if not isinstance(nodes, list):
            self.nodes = [nodes]
        elif isinstance(nodes, list):
            self.nodes = nodes

        self.combobox_node.clear()
        self.combobox_node.addItems([node.name for node in self.nodes])

    @gui.deferred
    def commit(self):
        """
        Commits the result the next widget in the line.
        """
        self.Outputs.output.send(self.output)


def _load_image(file):
    """
    Loads and create a DrbImageBaseNode from local file.
    Parameters:
        file (str): path to local file
    Returns:
        DrbImageBaseNode: a Drb Node of the TC Image
    """
    base_node = DrbFileFactory().create(file)
    image_node = DrbImageFactory().create(base_node)

    return image_node


if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview
    WidgetPreview(OWCloudMask).run()



from Orange.widgets import widget, gui, settings
from Orange.widgets.widget import Output
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QApplication, QStyle
from PyQt5.QtWidgets import QAction, QFileDialog, QPushButton
from drb.drivers.file import DrbFileFactory
from drb.drivers.image import DrbImageFactory, DrbImageBaseNode
import os
import drb.topics.resolver as resolver


class OWLoadImage(widget.OWWidget):
    name = "EO Load Image"
    description = "Loads image(s) from the directory structure"
    icon = "icons/load.svg"
    priority = 90

    current_path = None
    file = []
    image = []
    want_main_area = False
    graph_name = "thumbnailView"
    auto_commit = settings.Setting(True)

    class Outputs:
        output = Output("DrbImageBaseNode",
                        DrbImageBaseNode, auto_summary=False)

    def __init__(self):
        super().__init__()
        self.output = None

        browse_action_file = QAction(
            self,
            text="\N{HORIZONTAL ELLIPSIS}",
        )

        browse_action_file.triggered.connect(self.__show_open_dialog_file)
        browse_button_file = QPushButton(
            browse_action_file.iconText(),
            clicked=browse_action_file.trigger
        )

        vbox = gui.vBox(self.controlArea, box="Select product file:")
        hbox = gui.hBox(vbox)
        hbox.layout().addWidget(browse_button_file)

        gui.rubber(self.controlArea)
        self.combobox_file = gui.comboBox(
            vbox, self, "", items=self.file,
            contentsLength=12,
        )

        browse_action = QAction(
            self,
            text="\N{HORIZONTAL ELLIPSIS}",
        )

        browse_action.triggered.connect(self.__show_open_dialog_image)
        browse_button = QPushButton(
            browse_action.iconText(),
            clicked=browse_action.trigger
        )

        vbox = gui.vBox(self.controlArea, box="Load image:")
        hbox = gui.hBox(vbox)
        hbox.layout().addWidget(browse_button)

        gui.rubber(self.controlArea)
        self.combobox_image = gui.comboBox(
            vbox, self, "", items=self.image,
            contentsLength=12,
        )

        gui.rubber(self.controlArea)
        gui.auto_commit(self.buttonsArea, self,
                        "auto_commit", "Send", box=False)

    @gui.deferred
    def commit(self):
        """
        Commits the result the next widget in the line.
        """
        if self.current_path:
            self.Outputs.output.send(self.output)

        else:
            self.Outputs.output.send(None)

    def __show_open_dialog_image(self):
        image, f = QFileDialog.getOpenFileName(
            self, 'Select directory/file', 'some/default/path/',
            'Image files(*.tif *.tiff *.image '
            '*.jp2 *.png *.gif *.webp *.bmp *.jpeg *.jpg *.blx '
            ' *.kap *.dt0 *.dt1 *.bin *.gpkg *.mem)')

        self.current_path = image
        self.image = [os.path.basename(image)]
        self.combobox_image.clear()
        self.combobox_image.addItems([image for image in self.image])
        if self.current_path:
            self.output = _load_image(self.current_path)
            self.commit.deferred()

    def __show_open_dialog_file(self):
        filename, f = QFileDialog.getOpenFileName(
            self, 'Select directory/file', 'some/default/path/',
            'Product files(*.safe *.zip *.tar)')

        self.current_path = filename
        self.file = [os.path.basename(filename)]
        self.combobox_file.clear()
        self.combobox_file.addItems([file for file in self.file])
        if self.current_path:
            self.output = _load_file(self.current_path)
            self.commit.deferred()


def _load_image(file):
    base_node = DrbFileFactory().create(file)
    image_node = DrbImageFactory().create(base_node)

    return image_node


def _load_file(file):

    topic, file_node = resolver.resolve(file)

    return file_node


if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview
    WidgetPreview(OWLoadImage).run()



import cv2
import os
import rasterio
from Orange.widgets import widget, gui
from Orange.widgets.utils.signals import Input, Output
from PyQt5.QtCore import Qt, QRectF
from PyQt5.QtWidgets import (QGraphicsScene, QGraphicsView,
                             QGraphicsPixmapItem,
                             QGraphicsRectItem, QGraphicsItem)
from Orange.widgets import settings
from drb.drivers.file import DrbFileFactory
from drb.drivers.image import DrbImageFactory, DrbImageBaseNode


class OWCropImage(widget.OWWidget):
    name = "EO Crop Image"
    description = "Crops the image into a selected area"
    icon = "icons/crop.svg"
    priority = 70

    rectangle = None
    output_image = None

    reduction_value = settings.Setting(1)
    crop_size = 100
    x = 10
    y = 10

    want_main_area = True
    graph_name = "thumbnailView"
    auto_commit = settings.Setting(True)

    class Inputs:
        data = Input("DrbImageBaseNode",
                     DrbImageBaseNode, auto_summary=False)

    class Outputs:
        output = Output("DrbImageBaseNode",
                        DrbImageBaseNode, auto_summary=False)

    def __init__(self):
        super().__init__()
        self.node = None
        self.output = None

        slider1 = gui.hSlider(
            self.controlArea, self, "crop_size",
            box="Select Crop Size:", minValue=10, maxValue=100, step=10,
            callback=self._update_rect_size,
            width=200,
        )
        slider1.sliderReleased.connect(self._on_slider_release)

        slider2 = gui.hSlider(
            self.controlArea, self, "x",
            box="Horizontal Range X: ", minValue=0, maxValue=512, step=10,
            callback=self._update_rect_size,
            width=200,
        )
        slider2.sliderReleased.connect(self._on_slider_release)

        slider3 = gui.hSlider(
            self.controlArea, self, "y",
            box="Vertical Range Y:", minValue=0, maxValue=512, step=10,
            callback=self._update_rect_size,
            width=200,
        )
        slider3.sliderReleased.connect(self._on_slider_release)

        gui.rubber(self.controlArea)

        box = gui.widgetBox(self.controlArea)
        self.button = gui.button(
            box, self, "Crop Image",
            callback=self.crop_image
        )

        gui.rubber(self.controlArea)
        gui.auto_commit(self.buttonsArea, self,
                        "auto_commit", "Send", box=False)

        # Main area box
        gui.vBox(self.mainArea)
        self.main_box_scene = QGraphicsScene(self)
        self.main_box_view = QGraphicsView(self.main_box_scene)

        self.main_layout = self.mainArea.layout()
        self.main_layout.addWidget(self.main_box_view)

        self.crop_box_scene = QGraphicsScene(self)
        self.crop_box_view = QGraphicsView(self.crop_box_scene)

        self.crop_layout = self.mainArea.layout()
        self.crop_layout.addWidget(self.crop_box_view)

        self.mainArea.setMinimumWidth(100)

    def crop_image(self):
        """
        On slider release, shows a miniature of the cropped image.
        """
        with rasterio.open(os.path.join(os.path.dirname(
                self.path) + f'/{self.filename}.tif'), 'w',
                           **self.profile) as dst:
            dst.write(self.output_image)

        self.output = _load_image(os.path.dirname(
            self.path) + f'/{self.filename}.tif')
        self.commit.deferred()

    def _on_slider_release(self):
        """
        On slider release, shows a miniature of the cropped image.
        This function calculates the crop window in the original image.
        """
        self.path = self.node.path.path
        name = os.path.basename(self.path)
        self.filename = os.path.splitext(name)[0] + '-crop-image'

        ds = self.node.get_impl(rasterio.DatasetReader)
        self.profile = ds.meta.copy()

        # height and width in the original image
        h = self.y / self.reduction_h
        w = self.x / self.reduction_w

        h_n = (self.y + self.crop_size) / self.reduction_h
        w_n = (self.x + self.crop_size) / self.reduction_w

        h_p = (self.y - self.crop_size) / self.reduction_h
        w_p = (self.x - self.crop_size) / self.reduction_w

        if h_n > ds.shape[0] and w_n > ds.shape[1]:
            crop_image = ds.read(window=((h_p, h), (w_p, w)))

        elif h_n > ds.shape[0] and w_n < ds.shape[1]:
            crop_image = ds.read(window=((h_p, h), (w, w_n)))

        elif h_n < ds.shape[0] and w_n > ds.shape[1]:
            crop_image = ds.read(window=((h, h_n), (w_p, w)))

        else:
            crop_image = ds.read(window=((h, h_n), (w, w_n)))

        self.output_image = crop_image

        # show a PNG thumbnail of the cropped image
        crop_image = cv2.resize(crop_image[1, :, :], (256, 256))

        cv2.imwrite(os.path.join(os.path.dirname(
            self.path) + f'/{self.filename}.png'), crop_image)
        pic = QGraphicsPixmapItem()
        pic.setPixmap(QPixmap(os.path.join(os.path.dirname(
            self.path) + f'/{self.filename}.png')))
        self.crop_box_scene.addItem(pic)

    def _update_rect_size(self):
        """
        Modifies the size of the ROI rectangle upon slider modification.
        """
        if self.rectangle is not None:
            self.main_box_scene.removeItem(self.rectangle)
        if self.x + self.crop_size > 512 and self.y + self.crop_size > 512:
            self.rectangle = QGraphicsRectItem(
                QRectF(self.x - self.crop_size, self.y -
                       self.crop_size, self.crop_size, self.crop_size))
            self.rectangle.setPen(Qt.red)

        elif self.x + self.crop_size > 512 > self.y + self.crop_size:
            self.rectangle = QGraphicsRectItem(
                QRectF(self.x - self.crop_size, self.y,
                       self.crop_size, self.crop_size))
            self.rectangle.setPen(Qt.red)
        elif self.x + self.crop_size < 512 < self.y + self.crop_size:
            self.rectangle = QGraphicsRectItem(
                QRectF(self.x, self.y - self.crop_size,
                       self.crop_size, self.crop_size))
            self.rectangle.setPen(Qt.red)
        else:
            self.rectangle = QGraphicsRectItem(
                QRectF(self.x, self.y, self.crop_size, self.crop_size))
            self.rectangle.setPen(Qt.red)

        # self.rectangle.setFlag(QGraphicsItem.ItemIsMovable)
        self.main_box_scene.addItem(self.rectangle)

    def _show_image(self):
        """
        Show the original image in a reduced (512,512) size.
        This function initializes variables corresponding to height's
        and width's reduction rate.
        """
        ds = self.node.get_impl(rasterio.DatasetReader)
        full_image = ds.read(1)

        image = cv2.resize(full_image, (512, 512))

        # height's and width's reduction rate
        self.reduction_h = 512 / full_image.shape[0]
        self.reduction_w = 512 / full_image.shape[1]

        path = self.node.path.path
        name = os.path.basename(path)
        filename = os.path.splitext(name)[0]

        cv2.imwrite(os.path.join(os.path.dirname(path) +
                                 f'/{filename}.png'), image)

        # show a PNG thumbnail of the original image
        pic = QGraphicsPixmapItem()
        pic.setPixmap(QPixmap(os.path.join(
            os.path.dirname(path) + f'/{filename}.png')))
        self.main_box_scene.addItem(pic)

    @Inputs.data
    def set_data(self, node):

        self.node = node
        if self.node:
            self._show_image()
        self.commit.now()

    @gui.deferred
    def commit(self):
        """
        Commits the result the next widget in the line.
        """
        self.Outputs.output.send(self.output)


def _load_image(file):
    """
    Loads and create a DrbImageBaseNode from local file.
    Parameters:
        file (str): path to local file
    Returns:
        DrbImageBaseNode: a Drb Node of the TC Image.
    """
    base_node = DrbFileFactory().create(file)
    image_node = DrbImageFactory().create(base_node)

    return image_node


if __name__ == "__main__":
    from orangewidget.utils.widgetpreview import WidgetPreview
    WidgetPreview(OWCropImage).run()
